#include <stdio.h>
#include <stdlib.h>
#include "iplugin.h"



static unsigned char
shifts[] = {
    0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
  114,120, 53, 79, 96,109, 72,108, 70, 64, 76, 67,116, 74, 68, 87,
  111, 52, 75,119, 49, 34, 82, 81, 95, 65,112, 86,118,110,122,105,
   41, 57, 83, 43, 46,102, 40, 89, 38,103, 45, 50, 42,123, 91, 35,
  125, 55, 54, 66,124,126, 59, 47, 92, 71,115, 78, 88,107,106, 56,
   36,121,117,104,101,100, 69, 73, 99, 63, 94, 93, 39, 37, 61, 48,
   58,113, 32, 90, 44, 98, 60, 51, 33, 97, 62, 77, 84, 80, 85,223,
  225,216,187,166,229,189,222,188,141,249,148,200,184,136,248,190,
  199,170,181,204,138,232,218,183,255,234,220,247,213,203,226,193,
  174,172,228,252,217,201,131,230,197,211,145,238,161,179,160,212,
  207,221,254,173,202,146,224,151,140,196,205,130,135,133,143,246,
  192,159,244,239,185,168,215,144,139,165,180,157,147,186,214,176,
  227,231,219,169,175,156,206,198,129,164,150,210,154,177,134,127,
  182,128,158,208,162,132,167,209,149,241,153,251,237,236,171,195,
  243,233,253,240,194,250,191,155,142,137,245,235,163,242,178,152 };


/**
 *  This plugin is designed to display decoded CVS login information.
 *
 */
class CCVSPlugin: public IPlugin
{
 public:

  CCVSPlugin();
  ~CCVSPlugin();
  
  virtual void processPacket( const char *src, const char *dst, connection *currentConnection );
  virtual std::string getAuthor();
  virtual std::string getDescription();

private:
    char * scramble (char * str);
    char * descramble (char *str);

};


CCVSPlugin::CCVSPlugin()
{
}


CCVSPlugin::~CCVSPlugin()
{
}

void CCVSPlugin::processPacket( const char *src, const char *dst, connection *currentConnection )
{
    /*
     * Early termination.
     */
    if ( currentConnection->dport != 2401 )
    {
	return;
    }

    /*
     * Is this a login request?
     */
    char *begin = "BEGIN VERIFICATION REQUE";
    if ( strstr( currentConnection->buf, begin ) == NULL )
    {
	return;
    }

    /* Skip to next line. */
    char *ptr = currentConnection->buf + strlen( begin );
    while( ptr[0] != '\n' )
    {
	ptr +=1;
    }
    ptr += 1;

    std::string path;
    std::string user;
    std::string pass;

    while( ptr[0] != '\n' )
    {
	path += ptr[0];
	ptr +=1;
    }
    ptr += 1;

    while( ptr[0] != '\n' )
    {
	user += ptr[0];
	ptr +=1;
    }
    ptr += 1;

    while( ptr[0] != '\n' )
    {
	pass += ptr[0];
	ptr +=1;
    }
    ptr += 1;

    pass = descramble( (char *)pass.c_str() );
    if ( pass.size() )
    { 
	pass = "\"" + pass;
	pass += "\"";
    };
    printf("CVS: %s@%s:%s -> %s\n", user.c_str(), dst, path.c_str(), pass.c_str() );
}




/* SCRAMBLE and DESCRAMBLE work like this:
 *
 * scramble(STR) returns SCRM, a scrambled copy of STR.  SCRM[0] is a
 * single letter indicating the scrambling method.  As of this
 * writing, the only legal method is 'A', but check the code for more
 * up-to-date information.  The copy will have been allocated with
 * xmalloc(). 
 *
 * descramble(SCRM) returns STR, again in its own xmalloc'd space.
 * descramble() uses SCRM[0] to determine which method of unscrambling
 * to use.  If it does not recognize the method, it dies with error.
 */

/* Return a xmalloc'd, scrambled version of STR. */
char * CCVSPlugin::scramble (char * str)
{
    int i;
    char *s;

    /* +2 to hold the 'A' prefix that indicates which version of
       scrambling this is (the first, obviously, since we only do one
       kind of scrambling so far), and then the '\0' of course.  */
    s = (char *) malloc (strlen (str) + 2);

    /* Scramble (TM) version prefix. */
    s[0] = 'A';
    strcpy (s + 1, str);

    for (i = 1; s[i]; i++)
	s[i] = shifts[(unsigned char)(s[i])];

    return s;
}

/* Decode the string in place. */
char * CCVSPlugin::descramble (char *str)
{
    char *s;
    int i;

    /* For now we can only handle one kind of scrambling.  In the future
       there may be other kinds, and this `if' will become a `switch'.  */
    if (str[0] == 'A')
    {
	/* Method `A' is symmetrical, so scramble again to decrypt. */
	s = scramble (str + 1);

	/* Shift the whole string one char to the left, pushing the unwanted
	   'A' off the left end.  Safe, because s is null-terminated. */
	for (i = 0; s[i]; i++)
	    s[i] = s[i + 1];
	
	return s;
    }
    return( NULL );
}



std::string CCVSPlugin::getAuthor()
{
    return( "Steve Kemp" );
}

std::string CCVSPlugin::getDescription()
{
    return( "Sniff for CVS login information" );
}


/**
 * Create a new instance of this plugin object.
 */
extern "C" IPlugin *factory()
{
    return new CCVSPlugin();
};
